﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ProgrammingLanguagesConverter
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }


        List<string> sourceFile = new List<string>(); //коллекция строк входного файла
        List<string> outputFile = new List<string>();
        List<Lexem> sourceFileLexem = new List<Lexem>(); //коллекция исходных лексем
        List<Lexem> outputFileLexem = new List<Lexem>(); //коллекция конвертированных лексем
        ConvertableLanguage sourceLang = new ConvertableLanguage();
        ConvertableLanguage requiredLang = new ConvertableLanguage();
        bool bTextChanged = true;
        bool bSourceTypeLangChanged = true;
        bool bRequiredTypeLangChanged = true;

        //путь к исходному файлу
        string SourceFilePath
        {
            get { return sourcePathTextBox.Text; }
            set { sourcePathTextBox.Text = value; }
        }

        string SaveToFilePath
        {
            get;
            set;
        }

        private void sourcePathTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                SourceFilePath = sourcePathTextBox.Text;
        }

        private void sourcePathTextBox_Leave(object sender, EventArgs e)
        {
            SourceFilePath = sourcePathTextBox.Text;
        }

        private void syntaxHighlight(RichTextBox target, List<Lexem> source)
        {
            int startIndex = 0;
            foreach (Lexem lex in source)
            {
                if (lex.Type == LexemType.Keyword || lex.Type == LexemType.VarType)
                {
                    target.Select(startIndex, lex.Content.Length);
                    target.SelectionFont = new Font(target.Font, FontStyle.Bold);
                }

                if (lex.SubType == LexemSubType.programEnd)
                {
                    startIndex += 1;
                }

                startIndex += lex.Content.Length;                
            }
        }

        private void loadSourceFileButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog loadFileDialog = new OpenFileDialog(); //инициализация переменной диалога выбора файла
            loadFileDialog.Filter = "Файлы исходного кода|*.txt;*.pas;*.c;*.bas";

            if (loadFileDialog.ShowDialog() != DialogResult.OK)
                return;

            SourceFilePath = loadFileDialog.FileName;

            StreamReader sf = null; //вспомогательная переменная потока чтения
            sourceFile = new List<string>();

            try
            {
                sf = new StreamReader(SourceFilePath, Encoding.Default); //попытка чтения файла
            }
            catch
            {
                MessageBox.Show("Неверно указан путь к исходному файлу! Выбирите файл еще раз и повторите попытку.");
                SourceFilePath = "";
                return;
            }


            string bufStr = "";
            int j = SourceFilePath.Length-1;

            while (SourceFilePath[j] != '.')
            {
                bufStr = SourceFilePath[j] + bufStr;
                j--;
            }

            switch (bufStr.ToLower())
            {
                case "c":
                    sourceTypeComboBox.SelectedItem = sourceTypeComboBox.Items[0];
                    break;
                case "pas":
                    sourceTypeComboBox.SelectedItem = sourceTypeComboBox.Items[1];
                    break;
                case "bas":
                    sourceTypeComboBox.SelectedItem = sourceTypeComboBox.Items[2];
                    break;
                default:
                    break;
            }


            //считывание строк из файла в коллекцию
            while (true)
            {
                string str = sf.ReadLine();

                if (str == null)
                    break;

                sourceFile.Add(str);
            }

            sourceFileRichTextBox.Text = "";
          
            for (int i = 0; i < sourceFile.Count; i++)
            {
                sourceFileRichTextBox.Text += sourceFile[i] + "\n";
            }

            if (bSourceTypeLangChanged)
            {                
                switch (sourceTypeComboBox.SelectedItem.ToString())
                {
                    case "C":
                        sourceLang.LoadLexemsFromFile("C.xml");
                        break;
                    case "Pascal":
                        sourceLang.LoadLexemsFromFile("Pascal.xml");
                        break;
                    case "Basic":
                        sourceLang.LoadLexemsFromFile("Basic.xml");
                        break;
                }

                bSourceTypeLangChanged = false;
            }                       

            sourceFileLexem = LanguageConvertation.SyntaxAnalyzer(sourceLang.MainLexems, sourceFileRichTextBox.Lines.ToList<string>());
            //syntaxHighlight(sourceFileRichTextBox, sourceFileLexem);            
        }

        private void ConvertionButton_Click(object sender, EventArgs e)
        {
            convertedFileRichTextBox.Text = "";

            if (bSourceTypeLangChanged)
            {
                switch (sourceTypeComboBox.SelectedItem.ToString())
                {
                    case "C":
                        sourceLang.LoadLexemsFromFile("C.xml");
                        break;
                    case "Pascal":
                        sourceLang.LoadLexemsFromFile("Pascal.xml");
                        break;
                    case "Basic":
                        sourceLang.LoadLexemsFromFile("Basic.xml");
                        break;
                }

                bSourceTypeLangChanged = false;
            }

            if (bRequiredTypeLangChanged)
            {
                switch (convertionTypeComboBox.SelectedItem.ToString())
                {
                    case "C":
                        requiredLang.LoadLexemsFromFile("C.xml");
                        break;
                    case "Pascal":
                        requiredLang.LoadLexemsFromFile("Pascal.xml");
                        break;
                    case "Basic":
                        requiredLang.LoadLexemsFromFile("Basic.xml");
                        break;
                }

                bRequiredTypeLangChanged = false;
            }            

            if (sourceTypeComboBox.SelectedItem == convertionTypeComboBox.SelectedItem)
            {
                convertedFileRichTextBox.Text = sourceFileRichTextBox.Text;
                syntaxHighlight(convertedFileRichTextBox, sourceFileLexem);
                return;
            }

            if (bTextChanged)
            {
                sourceFileLexem.Clear();
                sourceFileLexem = LanguageConvertation.SyntaxAnalyzer(sourceLang.MainLexems, sourceFileRichTextBox.Lines.ToList<string>());
                bTextChanged = false;
                outputFileLexem.Clear();
                outputFileLexem = LanguageConvertation.LexemTranslate(sourceFileLexem, sourceLang, requiredLang);
                outputFile.Clear();
                foreach (Lexem lex in outputFileLexem)
                {
                    outputFile.Add(lex.Content);
                }
            }    

			foreach (Lexem lex in outputFileLexem)
			{
				convertedFileRichTextBox.Text += lex.Content;
			}

            //syntaxHighlight(convertedFileRichTextBox, outputFileLexem);
        }

        private void saveToFileButton_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog(); //инициализация переменной диалога выбора файла
            saveFileDialog.Filter = "Файлы исходного кода|*.txt;*.pas;*.c;*.bas";

            switch (convertionTypeComboBox.SelectedItem.ToString())
            {
                case "C":
                    saveFileDialog.FileName = "*.c";
                    break;
                case "Pascal":
                    saveFileDialog.FileName = "*.pas";
                    break;
                case "Basic":
                    saveFileDialog.FileName = "*.bas";
                    break;
            }            

            if (saveFileDialog.ShowDialog() != DialogResult.OK)
                return;

            SaveToFilePath = saveFileDialog.FileName;

            StreamWriter sw = null;
            sourceFile = new List<string>();

            try
            {
                sw = new StreamWriter(SaveToFilePath, false, Encoding.Default);
            }
            catch
            {
                MessageBox.Show("Неверно заданы параметры сохранения! Повторите попытку еще раз.");
                SaveToFilePath = "";
                return;

            }
            
            foreach (string str in outputFile)
            {
                if (str == "\n")
                {
                    sw.WriteLine();
                }
                else
                {
                    sw.Write(str);
                }
            }

            sw.Close();

            MessageBox.Show("Файл успешно сохранен");
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            convertionTypeComboBox.SelectedIndex = 0;
            sourceTypeComboBox.SelectedIndex = 0;
        }

        private void sourceFileRichTextBox_TextChanged(object sender, EventArgs e)
        {
            bTextChanged = true;
        }

        private void sourceTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            bSourceTypeLangChanged = true;
        }

        private void convertionTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            bRequiredTypeLangChanged = true;
            bTextChanged = true;
        }
    }
}
