﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Collections;

namespace ProgrammingLanguagesConverter
{
    //класс непосредственной конвертации
    static class LanguageConvertation
    {
        //функция, разбивающая входной текст на лексемы
        public static List<Lexem> SyntaxAnalyzer(List<Lexem> sourceLangLex, List<string> sourceFile)
        {
            int startLexemIndex = 0; 
            int endLexemIndex = 0; //индексаторы начала и конца лексемы
			string ambBufStr = null; //вспомогательная строка для определения однозначности лексемы
            List<Lexem> dividers = new List<Lexem>(); //коллекция разделителей
            List<Lexem> sourceFileLexem = new List<Lexem>(); //коллекция лексем входного файла

			//добавляем разделители из исходного языка в коллекцию
            foreach (Lexem lex in sourceLangLex)
            {
                if (lex.Type == LexemType.Divider)
                {
                    dividers.Add(lex);
                }
            }
			dividers.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));

			//проверяем каждую строку входного файла
            for (int i = 0; i < sourceFile.Count(); i++)
            {
                //добавляем символ перехода на новую строку
                sourceFile[i] += "\n";

                //в начале каждой строки присваиваем конечному и начальному индексу нулевое значение
                if (sourceFile[i].Length > 0 && sourceFile[i][0] == ' ')
                    startLexemIndex = 1;
                else
                    startLexemIndex = 0;
                endLexemIndex = 0;

                //проверяем строку посимвольно, пока не встретим разделитель
                for (int j = 0; j < sourceFile[i].Length; j++)
                {
                    //избавляемся от лишних пробелов
					if (j > 0 && sourceFile[i][j] == ' ' && sourceFile[i][j - 1] == ' ')
                    {
                        startLexemIndex = j + 1;
                        continue;
                    }

					//если данный символ содержится в списке разделителей
					if (CheckLexemContaining(sourceFile[i][j], dividers))
                    {
                        Lexem bufLexem = new Lexem(); //создаем лексему-буфер
                        endLexemIndex = j; //конечному индексу лексемы присваиваем значение текущего символа

                        //если данный символ не равен начальному индексу, добавляем новую лексему в коллекцию, иначе эта лексема и является разделителем
                        if (j != startLexemIndex)
                        {
							//создаем лексему-буфер на основе полученной подстроки
							bufLexem = FindLexemByName(sourceFile[i].Substring(startLexemIndex, endLexemIndex - startLexemIndex), sourceLangLex); 

							//проверяем буфер однозначности лексемы; 
							//если не пуст, ищем в списке лексем комбинированную, 
							//иначе добавляем только данную лексему
							if (ambBufStr != null)
							{
								Lexem ambBufLex = FindLexemByName(ambBufStr + bufLexem.Content, sourceLangLex);

								if (ambBufLex.Ambiguousity == "double")
								{
									sourceFileLexem.Add(ambBufLex);
								}
								else
								{
									sourceFileLexem.Add(FindLexemByName(ambBufStr, sourceLangLex));
									sourceFileLexem.Add(bufLexem);
								}

								ambBufStr = null;
							}
							else
							{
								sourceFileLexem.Add(bufLexem);
							}
                        }

                        //добавляем разделитель в список лексем
						bufLexem = FindLexemByName(new string(sourceFile[i][j], 1), dividers);

						//проверяем однозначность разделителя
						if (bufLexem.Ambiguousity == "single")
						{
							ambBufStr = bufLexem.Content;
						}
						else if (ambBufStr != null)
						{
							Lexem ambBufLex = FindLexemByName(ambBufStr + bufLexem.Content, sourceLangLex);

							if (ambBufLex.Ambiguousity == "double")
							{
								sourceFileLexem.Add(ambBufLex);
							}
							else
							{
								sourceFileLexem.Add(FindLexemByName(ambBufStr, sourceLangLex));
								sourceFileLexem.Add(bufLexem);
							}

							ambBufStr = null;
						}
						else
						{
                            if (bufLexem.SubType == LexemSubType.stringOpen)
                            {
                                bufLexem.Content = "";
                            }
							sourceFileLexem.Add(bufLexem);
						}
					
                        startLexemIndex = endLexemIndex = j + 1; //перебрасываем индексаторы на следующий символ
                    }
                }
            }

            return sourceFileLexem;
        }

		//функция непосредственного перевода коллекции лексем на исходном языке в коллекцию на необходимом
		public static List<Lexem> LexemTranslate(List<Lexem> sourceFileLexem, ConvertableLanguage sourceLang, ConvertableLanguage requiredLang)
		{
			List<Lexem> outputFileLexem = new List<Lexem>(); //коллекция выходных лексем

			List<Lexem> requiredLangLex = requiredLang.MainLexems; //коллекция лексем исходного языка

			bool bVariableReading = false; //флаг чтения переменной
		    int variableCursorPosition = 0;
			LexemSubType typeOfCurrentLexem = LexemSubType.any;
			List<Lexem> bufVariableLexems = new List<Lexem>();
            List<Lexem> bufGlobalVariableLexems = new List<Lexem>();
            int cursorPositionInProc = 0;

            bool bConstReading = false;
		    bool bConstValueReading = false;
            List<Lexem> bufConstValueLexems = new List<Lexem>();

            bool bNotTranslate = false;
            List<Lexem> bufNotTranslatedLexems = new List<Lexem>();

            bool bArrayReading = false;
            bool bCommaInArrayRead = false;
            bool bVarSwitchedToArr = false;
            int arrayCursorPosition = 0;
            int squareStamplesCounter = 0;
		    int stamplesOpenCounter = 0;
            int stamplesCloseCounter = 0;
            List<Lexem> bufArrayConstLexems = new List<Lexem>();

			bool bIfStateReading = false; //флаг чтения условной конструкции
            int cursorIfElse = 0;

			bool bForLoopReading = false; //флаг чтения цикла со счетчиком
			List<Lexem> bufForLoopLexems = new List<Lexem>();
            Stack<Lexem> bufBasicForLoopLexems = new Stack<Lexem>();
            int wasReadForLoopCount = 0;

			bool bWhileLoopReading = false;

            bool bRepeatLoopReading = false;
            bool bRepeatDefenitonReading = false;
            bool bCarriageReturnRead = false;
            
            bool bDoRead = false;

            string basicLoopRotation = "";

            bool bNameRead = false;
            bool bNeedChangeToArrayStamples = false;
                        
            LexemSubType subProgRead = LexemSubType.any;
            int blockOpenCloseCounter = 0;

            bool bSubProgHeaderReading = false;
		    Lexem currSubProgName = new Lexem();

            bool bMainConstructionReading = false;
            bool bMainConstructionInserted = false;

            bool bInputReading = false;
            bool bOutputReading = false;
            bool bQuotesRead = false;
            bool bEscapeRead = false;
            bool bLineFuncRead = false;
            bool bArrayInIORead = false;
            int quoteInIOCursor = 0;

            bool bPreProcReading = false;

            List<Lexem> allVars = new List<Lexem>();

			//проверяем каждую лексему в исходной коллекции
			foreach (Lexem lex in sourceFileLexem)
			{
				//если данная лексема является лексемой перехода на следующую строку переходим на следующую итерацию
				if (lex.Content == "\n" && sourceLang.Name != "Basic")
                {
                    if (bPreProcReading)
                    {
                        bPreProcReading = false;
                    }
					outputFileLexem.Add(lex);
					continue;
				}

                if (cursorIfElse > 0)
                {
                    if (lex.SubType == LexemSubType.algElse)
                    {
                        outputFileLexem.Insert(cursorIfElse, FindLexemWithSubType(LexemSubType.algElse, requiredLangLex));                        
                        cursorIfElse = -1;
                        basicLoopRotation += "i";
                        continue;
                    }
                    else
                    {
                        outputFileLexem.Insert(cursorIfElse, FindLexemWithSubType(LexemSubType.algEnd, requiredLangLex));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algIf, requiredLangLex));
                        cursorIfElse = 0;
                    }
                }

				//считаем количество открытых скобок
                if (lex.SubType == LexemSubType.squareStampleOpen)
				{
					squareStamplesCounter++;
				}
                else if (lex.SubType == LexemSubType.stampleOpenSymbol)
                {
                    stamplesOpenCounter++;
                }
                else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                {
                    stamplesCloseCounter++;
                }
                else if (lex.SubType == LexemSubType.blockStart)
                {
                    squareStamplesCounter = 0;
                    blockOpenCloseCounter++;
                }
                else if (lex.SubType == LexemSubType.blockEnd)
                {
                    blockOpenCloseCounter--;
                }

                if (bMainConstructionReading)
                {
                    if (lex.SubType == LexemSubType.blockStart)
                    {
                        bMainConstructionReading = false;
                        bVariableReading = false;
                        outputFileLexem.Add(FindLexemWithSubType(lex.SubType, requiredLangLex));
                    }
                    else if (lex.SubType == LexemSubType.endOfLine)
                    {
                        bMainConstructionReading = false;
                    }

                    continue;
                }
                else if (bPreProcReading)
                {
                    continue;
                }

                if (sourceLang.Name == "Basic" && !bMainConstructionInserted)
                {
                    if (requiredLang.Name == "C")          
                    {
                        if (lex.SubType != LexemSubType.function && lex.SubType != LexemSubType.procedure)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.typeVoid, requiredLangLex));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.programStart, requiredLangLex));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            if (bufGlobalVariableLexems.Count > 0)
                            {
                                foreach (Lexem blex in bufGlobalVariableLexems)
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(blex.SubType, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Add(new Lexem(blex.Content, LexemType.Variable, LexemSubType.variable));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                }
                                bufGlobalVariableLexems.Clear();
                            }
                            cursorPositionInProc = outputFileLexem.Count;
                            bMainConstructionInserted = true;
                        }
                    }
                    else if (requiredLang.Name == "Pascal")
                    {
                        if (lex.SubType != LexemSubType.varBlock && lex.SubType != LexemSubType.procedure)
                        {
                            outputFileLexem.Insert(variableCursorPosition, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                            outputFileLexem.Insert(variableCursorPosition, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            bMainConstructionInserted = true;
                        }
                    }
                }

				//если данная лексема является переменной
                if (lex.Type == LexemType.Variable)
                {
                    if ((bNameRead || bIfStateReading) && sourceLang.Name == "Basic")
                    {
                        foreach (Lexem itm in allVars)
                        {
                            if (itm.Content == lex.Content)
                            {
                                bArrayInIORead = true;
                                break;
                            }
                        }
                    }

                    Lexem blex = lex;

                    if (typeOfCurrentLexem != LexemSubType.any)
                    {
                        blex.SubType = typeOfCurrentLexem;
                    }
                    allVars.Add(blex);

                    if (bVariableReading)
                    {
                        if (!bConstValueReading)
                        {
                            if (subProgRead == LexemSubType.any && sourceLang.Name == "Pascal" && requiredLang.Name == "C")
                                bufGlobalVariableLexems.Add(lex);
                            else
                                bufVariableLexems.Add(lex);
                        }
                    }
                    else if (bArrayReading)
                    {
                        if (subProgRead == LexemSubType.any && sourceLang.Name == "Pascal" && requiredLang.Name == "C" && !bVarSwitchedToArr)
                        {
                            bVarSwitchedToArr = true;
                            for (int i = 0; i < bufGlobalVariableLexems.Count; i++)
                            {
                                Lexem plex = bufGlobalVariableLexems[i];
                                plex.Content = "<" + bufGlobalVariableLexems[i].Content;
                                bufGlobalVariableLexems.RemoveAt(i);
                                bufGlobalVariableLexems.Insert(i, plex);
                            }
                        }

                        if (lex.SubType == LexemSubType.constant)
                        {
                            if (bufGlobalVariableLexems.Count > 0)
                            {
                                for (int i = 0; i < bufGlobalVariableLexems.Count; i++)
                                {
                                    if (bufGlobalVariableLexems[i].Content[0] == '<')
                                    {
                                        if (!Char.IsDigit(bufGlobalVariableLexems[i].Content[1]))
                                        {
                                            Lexem plex = bufGlobalVariableLexems[i];
                                            plex.Content = plex.Content.Insert(1, lex.Content);
                                            bufGlobalVariableLexems.RemoveAt(i);
                                            bufGlobalVariableLexems.Insert(i, plex);
                                        }
                                        else if (!Char.IsDigit(bufGlobalVariableLexems[i].Content[2]))
                                        {
                                            Lexem plex = bufGlobalVariableLexems[i];
                                            plex.Content = plex.Content.Insert(2, lex.Content);
                                            bufGlobalVariableLexems.RemoveAt(i);
                                            bufGlobalVariableLexems.Insert(i, plex);
                                        }
                                    }
                                }
                            }

                            if (sourceLang.BGapInArr && !requiredLang.BGapInArr && bufArrayConstLexems.Count > 0)
                            {
                                int bufCont = System.Convert.ToInt32(lex.Content) - System.Convert.ToInt32(bufArrayConstLexems[0].Content) + 1;
                                bufArrayConstLexems.Clear();
                                bufArrayConstLexems.Add(new Lexem(bufCont.ToString(), LexemType.Variable, LexemSubType.constant));
                                continue;
                            }

                            bufArrayConstLexems.Add(lex);
                        }
                        else
                        {
                            bufVariableLexems.Add(lex);
                        }
                    }
                    else if (bForLoopReading)
                    {
                        Lexem bufLex = lex;

                        if (CheckLexemContaining(LexemSubType.variable, bufForLoopLexems))
                        {
                            if (FindLexemWithSubType(LexemSubType.variable, bufForLoopLexems).Content == lex.Content) continue;

                            bufLex.SubType = LexemSubType.constant;
                        }

                        bufForLoopLexems.Add(bufLex);
                    }
                    else if (bSubProgHeaderReading)
                    {
                        bufVariableLexems.Add(lex);
                    }
                    else if (wasReadForLoopCount == -2)
                    {
                        wasReadForLoopCount = 0;
                        continue;
                    }
                    else if (bInputReading)
                    {
                        foreach (Lexem itm in allVars)
                        {
                            if (itm.Content == lex.Content)
                            {
                                bArrayInIORead = true;
                                break;
                            }
                        }

                        if (requiredLang.Name == "C")
                        {
                            if (!bQuotesRead && squareStamplesCounter == 0)
                            {
                                if ((stamplesOpenCounter == 0 && sourceLang.Name == "Basic") || sourceLang.Name != "Basic")
                                {
                                    LexemSubType lsto = LexemSubType.any;

                                    for (int i = 0; i < allVars.Count; i++)
                                    {
                                        if (allVars[i].Content == lex.Content)
                                        {
                                            lsto = allVars[i].SubType;
                                            break;
                                        }
                                    }

                                    switch (lsto)
                                    {
                                        case LexemSubType.typeString:
                                            outputFileLexem.Add(new Lexem("\"%s\", ", LexemType.Variable, LexemSubType.any));
                                            break;
                                        case LexemSubType.type32sInt:
                                            outputFileLexem.Add(new Lexem("\"%i\", &", LexemType.Variable, LexemSubType.any));
                                            break;
                                        default:
                                            MessageBox.Show("fd");
                                            break;
                                    }
                                }
                            }
                        }

                        outputFileLexem.Add(lex);
                    }
                    else if (bOutputReading)
                    {
                        foreach (Lexem itm in allVars)
                        {
                            if (itm.Content == lex.Content)
                            {
                                bArrayInIORead = true;
                                break;
                            }
                        }

                        if (requiredLang.Name == "C")
                        {
                            if (!bQuotesRead && squareStamplesCounter == 0)
                            {
                                LexemSubType lsto = LexemSubType.any;

                                for (int i = 0; i < allVars.Count; i++)
                                {
                                    if (allVars[i].Content == lex.Content)
                                    {
                                        lsto = allVars[i].SubType;
                                        break;
                                    }
                                }

                                if (quoteInIOCursor > 0)
                                {
                                    switch (lsto)
                                    {
                                        case LexemSubType.typeString:
                                            outputFileLexem.Insert(quoteInIOCursor, new Lexem("%s", LexemType.Variable, LexemSubType.any));
                                            break;
                                        case LexemSubType.type32sInt:
                                            outputFileLexem.Insert(quoteInIOCursor, new Lexem("\"%i", LexemType.Variable, LexemSubType.any));
                                            break;
                                    }

                                    if (bLineFuncRead)
                                    {
                                        outputFileLexem.Insert(quoteInIOCursor + 1, new Lexem(" \\n", LexemType.Variable, LexemSubType.any));
                                    }
                                }
                                else
                                {
                                    switch (lsto)
                                    {
                                        case LexemSubType.typeString:
                                            outputFileLexem.Add(new Lexem("\"%s", LexemType.Variable, LexemSubType.any));
                                            break;
                                        case LexemSubType.type32sInt:
                                            outputFileLexem.Add(new Lexem("\"%i", LexemType.Variable, LexemSubType.any));
                                            break;
                                    }

                                    if (bLineFuncRead)
                                    {
                                        outputFileLexem.Add(new Lexem(" \\n\", ", LexemType.Variable, LexemSubType.any));
                                    }
                                    else
                                    {
                                        outputFileLexem.Add(new Lexem("\", ", LexemType.Variable, LexemSubType.any));
                                    }
                                }

                            }
                        }

                        if (sourceLang.Name == "C")
                        {
                            if (lex.Content[0] == '%')
                            {
                                continue;
                            }
                        }

                        outputFileLexem.Add(lex);
                    }
                    else if (!bMainConstructionReading && !bInputReading)
                    {
                        if (!bNameRead)
                        {
                            bNameRead = true;
                            squareStamplesCounter = 0;
                        }
                        outputFileLexem.Add(lex);
                    }

                    continue;
                }
                
                if (lex.Type == LexemType.VarType)
				{
				   
                    typeOfCurrentLexem = lex.SubType;
				   
				    if (sourceLang.Name == "C")
				    {
				        bVariableReading = true;
                        variableCursorPosition = cursorPositionInProc;
				    }
                    else 
                    {
                        for (int i = 0; i < allVars.Count; i++)
                        {
                            if (allVars[i].SubType == LexemSubType.variable)
                            {
                                Lexem bufl = allVars[i];
                                bufl.SubType = typeOfCurrentLexem;
                                allVars.RemoveAt(i);
                                allVars.Insert(i, bufl);
                            }
                        }

                        if (subProgRead == LexemSubType.any)
                        {
                            for (int y = 0; y < bufGlobalVariableLexems.Count; y++)
                            {
                                if (bufGlobalVariableLexems[y].Type != LexemType.VarType)
                                {
                                    Lexem blx = bufGlobalVariableLexems[y];
                                    bufGlobalVariableLexems.RemoveAt(y);
                                    blx.SubType = lex.SubType;
                                    blx.Type = LexemType.VarType;
                                    bufGlobalVariableLexems.Insert(y, blx);
                                }
                            }
                        }
                    }
                    continue;
				}
				//если данная лексема является началом цикла со счетчиком, чекаем соответствующий флаг
				else if (lex.SubType == LexemSubType.algFor)
				{
					bForLoopReading = true;
					continue;
				}
					//аналогично для условной конструкции
				else if (lex.SubType == LexemSubType.algIf)
				{
                    if (bCommaInArrayRead)
                    {
                        bCommaInArrayRead = false;
                        continue;
                    }

					bIfStateReading = true;

					outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algIf, requiredLangLex));

                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
					if (requiredLang.BStrictlyStamples)
					{						
						outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
					}                    

					continue;
				}
                else if (lex.SubType == LexemSubType.algEnd)
                {
                    bCommaInArrayRead = true; //флажок прочтения AlgEnd
                    outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                    outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                    continue;
                }
                else if (lex.SubType == LexemSubType.algWhile)
                {
                    if (bRepeatLoopReading)
                    {
                        if (requiredLang.BStrictlyBlockInRepeat)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.any));
                        }

                        bRepeatDefenitonReading = true;

                        if (requiredLang.Name == "Basic")
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algLoop, requiredLangLex));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                        }
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algUntil, requiredLangLex));

                        if (requiredLang.BStrictlyStamples)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                        }

                        continue;
                    }

                    bWhileLoopReading = true;

                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algWhile, requiredLangLex));

                    if (requiredLang.BStrictlyStamples)
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                    }

                    if (sourceLang.Name == "Basic") bDoRead = false;

                    continue;
                }
                else if (lex.SubType == LexemSubType.algRepeat)
                {
                    bRepeatLoopReading = true;

                    if (requiredLang.Name == "Basic")
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algDo, requiredLangLex));
                    else
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algRepeat, requiredLangLex));

                    if (requiredLang.BStrictlyBlockInRepeat)
                    {
                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.any));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                    }

                    continue;
                }
                else if (lex.SubType == LexemSubType.algUntil)
                {
                    if (requiredLang.BStrictlyBlockInRepeat)
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.any));
                    }

                    if (requiredLang.Name == "Basic")
                    {                        
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algLoop, requiredLangLex));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                    }
                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algUntil, requiredLangLex));

                    if (requiredLang.BStrictlyStamples)
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                    }

                    bRepeatDefenitonReading = true;

                    continue;
                }
                else if (lex.SubType == LexemSubType.algDo)
                {
                    if (sourceLang.Name == "Basic")
                    {
                        bDoRead = true;
                        continue;
                    }
                }
                else if (lex.SubType == LexemSubType.varBlock)
				{
                    bConstReading = false;
					bVariableReading = true;
                    variableCursorPosition = cursorPositionInProc;
					continue;
				}
                else if (lex.SubType == LexemSubType.constDef)
                {
                    bConstReading = true;
                    bVariableReading = true;
                    variableCursorPosition = cursorPositionInProc;
                    continue;
                }
                else if (lex.SubType == LexemSubType.algWend)
                {
                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                    continue;
                }
                else if (lex.SubType == LexemSubType.endOfLine)
                {
                    bInputReading = false;
                    bOutputReading = false;
                    quoteInIOCursor = 0;

                    if (bVariableReading)
                    {
                        if (sourceLang.Name == "C")
                        {
                            if (requiredLang.Name == "Pascal")
                            {
                                foreach (Lexem bufLex in bufVariableLexems)
                                {
                                    outputFileLexem.Insert(variableCursorPosition++, bufLex);
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                }
                            }
                            else if (requiredLang.Name == "Basic")
                            {
                                foreach (Lexem bufLex in bufVariableLexems)
                                {
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.varBlock, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, bufLex);
                                    outputFileLexem.Insert(variableCursorPosition++, new Lexem("\n", LexemType.Divider, LexemSubType.any));

                                }
                            }

                            typeOfCurrentLexem = LexemSubType.any;
                            bVariableReading = false;
                            bufVariableLexems.Clear();
                            continue;
                        }
                        else if (sourceLang.Name == "Pascal")
                        {
                            if (requiredLang.Name == "C" && subProgRead != LexemSubType.any)
                            {
                                int i = variableCursorPosition;
                                if (bConstReading)
                                {
                                    outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.constDef, requiredLangLex));
                                    outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                }
                                outputFileLexem.Insert(i++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                for (int j = 0; j < bufVariableLexems.Count; j++)
                                {
                                    outputFileLexem.Insert(i, bufVariableLexems[j]);
                                    i++;

                                    if (j == bufVariableLexems.Count - 1) continue;

                                    outputFileLexem.Insert(i, FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                    i++;
                                    outputFileLexem.Insert(i, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    i++;
                                }
                                if (bConstReading)
                                {
                                    outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.logicAssign, requiredLangLex));
                                    outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    bufConstValueLexems.Reverse();
                                    foreach (Lexem bflex in bufConstValueLexems)
                                    {
                                        outputFileLexem.Insert(i++, bflex);
                                    }
                                }

                                outputFileLexem.Insert(i++, FindLexemWithSubType(lex.SubType, requiredLangLex));
                                outputFileLexem.Insert(i, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            }
                            else if (requiredLang.Name == "Basic")
                            {
                                foreach (Lexem bufLex in bufVariableLexems)
                                {
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.varBlock, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, bufLex);
                                    outputFileLexem.Insert(variableCursorPosition++, new Lexem("\n", LexemType.Divider, LexemSubType.any));

                                }
                            }

                            bConstValueReading = false;
                            bufConstValueLexems.Clear();
                            bufVariableLexems.Clear();
                            continue;
                        }

                        variableCursorPosition = cursorPositionInProc;
                    }
                    else if (bArrayReading)
                    {
                        if (!bCommaInArrayRead)
                        {
                            foreach (Lexem bufLex in bufVariableLexems)
                            {
                                switch (requiredLang.Name)
                                {
                                    case "C":
                                        {
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, bufLex);
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, bufArrayConstLexems[0]);
                                            outputFileLexem.Insert(variableCursorPosition++, bufArrayConstLexems[1]);
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        }
                                        break;

                                    case "Pascal":
                                        {
                                            outputFileLexem.Insert(variableCursorPosition++, bufLex);
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.arrayType, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.arrayRange, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, bufArrayConstLexems[0]);
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.algOf, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        }
                                        break;

                                    case "Basic":
                                        {
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.varBlock, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, bufLex);
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.algTo, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, bufArrayConstLexems[bufArrayConstLexems.Count - 1]);
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                        }
                                        break;
                                }
                            }
                        }
                        else
                        {
                            switch (requiredLang.Name)
                            {
                                case "C":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[1]);
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        outputFileLexem.Insert(cursorPositionInProc + 1, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    }
                                    break;

                                case "Pascal":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.arrayRange, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[0]);
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.algOf, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    }
                                    break;

                                case "Basic":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.algTo, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[bufArrayConstLexems.Count - 1]);
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                        outputFileLexem.Insert(cursorPositionInProc + 4, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        outputFileLexem.Insert(cursorPositionInProc + 5, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    }
                                    break;
                            }

                            squareStamplesCounter = 0;
                            arrayCursorPosition = cursorPositionInProc;
                            bCommaInArrayRead = false;
                        }

                        bArrayReading = false;
                        if (sourceLang.Name == "Pascal")
                        {
                            bVariableReading = true;
                            variableCursorPosition = cursorPositionInProc;
                        }
                        bufVariableLexems.Clear();
                        bufArrayConstLexems.Clear();
                        continue;
                    }
                    else if (bRepeatDefenitonReading)
                    {
                        if (requiredLang.BStrictlyStamples)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                        }

                        bRepeatLoopReading = false;
                        bCarriageReturnRead = false;
                        bRepeatDefenitonReading = false;
                    }
                    else if (bMainConstructionReading)
                    {
                        bMainConstructionReading = false;
                        continue;
                    }
                    
                    bNameRead = false;
                }
                else if (lex.SubType == LexemSubType.carriageReturn)
                {
                    bNameRead = false;

                    if (bInputReading || bOutputReading)
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                        bInputReading = false;
                        bOutputReading = false;
                        quoteInIOCursor = 0;
                    }                  

                    if (bVariableReading)
                    {
                        if (requiredLang.Name == "Pascal")
                        {
                            foreach (Lexem bufLex in bufVariableLexems)
                            {
                                outputFileLexem.Insert(variableCursorPosition++, bufLex);
                                outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            }
                        }
                        else if (requiredLang.Name == "C")
                        {
                            int i = variableCursorPosition;
                            if (bConstReading)
                            {
                                outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.constDef, requiredLangLex));
                                outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            }
                            outputFileLexem.Insert(i++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                            outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            for (int j = 0; j < bufVariableLexems.Count; j++)
                            {
                                outputFileLexem.Insert(i, bufVariableLexems[j]);
                                i++;

                                if (j == bufVariableLexems.Count - 1) continue;

                                outputFileLexem.Insert(i, FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                i++;
                                outputFileLexem.Insert(i, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                i++;
                            }
                            if (bConstReading)
                            {
                                outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.logicAssign, requiredLangLex));
                                outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                bufConstValueLexems.Reverse();
                                foreach (Lexem bflex in bufConstValueLexems)
                                {
                                    outputFileLexem.Insert(i++, bflex);
                                }
                            }

                            outputFileLexem.Insert(i++, FindLexemWithSubType(lex.SubType, requiredLangLex));
                            outputFileLexem.Insert(i++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                        }

                        bVariableReading = false;
                        bufVariableLexems.Clear();
                        outputFileLexem.Add(lex);
                        continue;
                    }
                    else if (bArrayReading)
                    {
                        if (!bCommaInArrayRead)
                        {
                            foreach (Lexem bufLex in bufVariableLexems)
                            {
                                switch (requiredLang.Name)
                                {
                                    case "C":
                                        {
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, bufLex);
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                            Lexem qlex = new Lexem
                                            {
                                                Content = (Convert.ToInt32(bufArrayConstLexems[0].Content.ToString()) + Convert.ToInt32(bufArrayConstLexems[1].Content.ToString())).ToString(),
                                                Type = LexemType.Variable,
                                                SubType = LexemSubType.constant,
                                                Ambiguousity = "single"
                                            };                 
                                            outputFileLexem.Insert(arrayCursorPosition++, qlex);
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        }
                                        break;

                                    case "Pascal":
                                        {
                                            outputFileLexem.Insert(arrayCursorPosition++, bufLex);
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.arrayType, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.arrayRange, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[1]);
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.algOf, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                            outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        }
                                        break;
                                }
                            }
                        }
                        else
                        {
                            switch (requiredLang.Name)
                            {
                                case "C":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[1]);
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition + 1, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    }
                                    break;

                                case "Pascal":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.arrayRange, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[bufArrayConstLexems.Count - 1]);
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.algOf, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    }
                                    break;
                            }

                            squareStamplesCounter = 0;
                            arrayCursorPosition = cursorPositionInProc;
                            bCommaInArrayRead = false;
                        }

                        bArrayReading = false;
                        bufVariableLexems.Clear();
                        bufArrayConstLexems.Clear();
                        outputFileLexem.Add(lex);
                        continue;
                    }
                    else if (bWhileLoopReading)
                    {
                        if (requiredLang.Name == "C")
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        }
                        else if (requiredLang.Name == "Pascal")
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algDo, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        }
                        bWhileLoopReading = false;
                    }
                    else if (bDoRead)
                    {
                        if (outputFileLexem.Count > 0 &&
                            (outputFileLexem[outputFileLexem.Count - 1].Type != LexemType.Divider &&
                            outputFileLexem[outputFileLexem.Count - 1].SubType != LexemSubType.spaceSymbol &&
                            outputFileLexem[outputFileLexem.Count - 1].Content != "\n" &&
                            outputFileLexem[outputFileLexem.Count - 1].SubType != LexemSubType.blockStart &&
                            outputFileLexem[outputFileLexem.Count - 1].SubType != LexemSubType.blockEnd &&
                            outputFileLexem[outputFileLexem.Count - 1].SubType != LexemSubType.algElse)
                           )
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                        }

                        outputFileLexem.Add(lex);
                        if (!bCarriageReturnRead && requiredLang.BStrictlyBlockInRepeat)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algRepeat, requiredLangLex));
                            outputFileLexem.Add(lex);
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                            outputFileLexem.Add(lex);
                            bCarriageReturnRead = true;
                        }

                        continue;
                    }
                    else if (bRepeatDefenitonReading)
                    {
                        if (requiredLang.BStrictlyStamples)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                        }

                        bRepeatLoopReading = false;
                        bCarriageReturnRead = false;
                        bRepeatDefenitonReading = false;
                    }

                    if (outputFileLexem.Count > 0 &&
                        (outputFileLexem[outputFileLexem.Count - 1].Type != LexemType.Divider &&
                        outputFileLexem[outputFileLexem.Count - 1].SubType != LexemSubType.spaceSymbol &&
                        outputFileLexem[outputFileLexem.Count - 1].Content != "\n" &&
                        outputFileLexem[outputFileLexem.Count - 1].Content != "" &&
                        outputFileLexem[outputFileLexem.Count - 1].SubType != LexemSubType.blockStart &&
                        outputFileLexem[outputFileLexem.Count - 1].SubType != LexemSubType.blockEnd &&
                        outputFileLexem[outputFileLexem.Count - 1].SubType != LexemSubType.algElse
                        )
                       )
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                    }

                    outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                }
                else if (lex.SubType == LexemSubType.blockStart)
                {
                    if (sourceLang.Name == "Pascal")
                    {
                        bVariableReading = false;
                        bConstReading = false;

                        if (requiredLang.Name == "C" && blockOpenCloseCounter == 1)
                        {
                            if (subProgRead == LexemSubType.any)
                            {
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.typeVoid, requiredLangLex));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.programStart, requiredLangLex));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                outputFileLexem.Add(FindLexemWithSubType(lex.SubType, requiredLangLex));
                                outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                if (bufGlobalVariableLexems.Count > 0)
                                {
                                    foreach (Lexem blex in bufGlobalVariableLexems)
                                    {
                                        if (blex.Content[0] == '<')
                                        {
                                            Lexem qlex = new Lexem();
                                            qlex.Content = blex.Content.Substring(3);
                                            outputFileLexem.Add(FindLexemWithSubType(blex.SubType, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Add(new Lexem(qlex.Content, LexemType.Variable, LexemSubType.variable));                                            
                                            qlex = new Lexem
                                            {
                                                Content = (Convert.ToInt32(blex.Content[1].ToString()) + Convert.ToInt32(blex.Content[2].ToString())).ToString(),
                                                Type = LexemType.Variable,
                                                SubType = LexemSubType.constant,
                                                Ambiguousity = "single"
                                            };                                            
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                            outputFileLexem.Add(qlex);
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                        }
                                        else
                                        {
                                            outputFileLexem.Add(FindLexemWithSubType(blex.SubType, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Add(new Lexem(blex.Content, LexemType.Variable, LexemSubType.variable));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                        }
                                    }
                                    bufGlobalVariableLexems.Clear();
                                }
                                cursorPositionInProc = outputFileLexem.Count + 1;                                
                                continue;
                            }
                            else
                            {
                                outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(lex.SubType, requiredLangLex));
                                outputFileLexem.Insert(variableCursorPosition, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                continue;
                            }
                        }
                    }
                }
                else if (lex.SubType == LexemSubType.blockEnd)
                {
                    if (subProgRead != LexemSubType.any && blockOpenCloseCounter == 0)
                    {
                        if (requiredLang.Name == "Basic")
                        {                            
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algEnd, requiredLangLex));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            if (subProgRead == LexemSubType.typeVoid)
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.procedure, requiredLangLex));
                            else
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.function, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));

                            cursorPositionInProc = outputFileLexem.Count + 1;
                        }
                        else if (requiredLang.Name == "Pascal")
                        {
                            cursorPositionInProc = 0;
                        }

                        currSubProgName = new Lexem();
                        subProgRead = LexemSubType.any;
                    }

                    if (basicLoopRotation.Length > 0)
                    {
                        switch (basicLoopRotation[basicLoopRotation.Length - 1])
                        {
                            case 'i':
                                if (cursorIfElse == -1)
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algEnd, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algIf, requiredLangLex));
                                    cursorIfElse = 0;
                                }
                                else
                                {
                                    cursorIfElse = outputFileLexem.Count;
                                }
                                basicLoopRotation = basicLoopRotation.Substring(0, basicLoopRotation.Length - 1);
                                continue;
                                break;

                            case 'f':
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algNext, requiredLangLex));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                outputFileLexem.Add(bufBasicForLoopLexems.Pop());
                                basicLoopRotation = basicLoopRotation.Substring(0, basicLoopRotation.Length - 1);
                                continue;
                                break;

                            case 'w':
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algWend, requiredLangLex));
                                outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                basicLoopRotation = basicLoopRotation.Substring(0, basicLoopRotation.Length - 1);
                                continue;
                                break;
                        }
                    }
                }
                else if (lex.SubType == LexemSubType.mathEqual)
                {
                    if (requiredLang.Name == "Basic")
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAssign, requiredLangLex));
                        continue;
                    }
                }
                else if (lex.SubType == LexemSubType.algNext)
                {
                    if (wasReadForLoopCount == -1)
                    {
                        wasReadForLoopCount = -2;
                    }

                    outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                    outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                    continue;
                }
                else if (lex.SubType == LexemSubType.algLoop)
                {
                    if (bDoRead)
                    {
                        bDoRead = false;
                        bRepeatLoopReading = true;
                    }
                    else
                    {
                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        continue;
                    }
                }
                else if (lex.SubType == LexemSubType.procedure || lex.SubType == LexemSubType.function)
                {
                    if (bCommaInArrayRead)
                    {
                        if (requiredLang.Name == "C")
                            cursorPositionInProc = outputFileLexem.Count;
                        else
                            cursorPositionInProc = 0;

                        blockOpenCloseCounter = 0;                        
                        continue;
                    }

                    if (requiredLang.Name == "Pascal")
                    {
                        variableCursorPosition = cursorPositionInProc;
                        cursorPositionInProc = 0;
                        outputFileLexem.Insert(variableCursorPosition++, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(lex.SubType, requiredLangLex));
                    }

                    bVariableReading = false;
                    bSubProgHeaderReading = true;
                    typeOfCurrentLexem = LexemSubType.any;
                    stamplesOpenCounter = 0;
                    continue;
                }
                else if (lex.SubType == LexemSubType.programStart)
                {
                    bMainConstructionReading = true;
                    continue;
                }
                else if (lex.SubType == LexemSubType.returnWord)
                {
                    if (requiredLang.Name == "Pascal")
                    {
                        outputFileLexem.Add(currSubProgName);
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAssign, requiredLangLex));                        
                        continue;
                    }
                }
                else if (lex.SubType == LexemSubType.logicAssign)
                {
                    if (requiredLang.Name == "C")
                    {
                        if (outputFileLexem[outputFileLexem.Count - 1].Content == currSubProgName.Content)
                        {
                            outputFileLexem.RemoveAt(outputFileLexem.Count - 1);
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.returnWord, requiredLangLex));
                            currSubProgName = new Lexem();
                            continue;
                        }
                        else if (outputFileLexem[outputFileLexem.Count - 2].Content == currSubProgName.Content)
                        {
                            outputFileLexem.RemoveAt(outputFileLexem.Count - 2);
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.returnWord, requiredLangLex));
                            currSubProgName = new Lexem();
                            continue;
                        }
                    }
                }
                else if (lex.SubType == LexemSubType.algElse)
                {
                    if (sourceLang.Name == "Basic")
                    {
                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        outputFileLexem.Add(FindLexemWithSubType(lex.SubType, requiredLangLex));
                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        continue;
                    }
                }
                else if (lex.SubType == LexemSubType.write || lex.SubType == LexemSubType.writeln)
                {
                    squareStamplesCounter = 0;
                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.write, requiredLangLex));
                    bOutputReading = true;
                    if (lex.SubType == LexemSubType.writeln)
                    {
                        bLineFuncRead = true;
                    }
                    stamplesOpenCounter = 0;
                    stamplesCloseCounter = 0;
                    if (sourceLang.Name == "Basic")
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                    }

                    continue;
                }
                else if (lex.SubType == LexemSubType.read || lex.SubType == LexemSubType.readln)
                {
                    squareStamplesCounter = 0;
                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.read, requiredLangLex));                    
                    bInputReading = true;
                    stamplesOpenCounter = 0;
                    stamplesCloseCounter = 0;
                    if (sourceLang.Name == "Basic")
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                    }
                    continue;
                }
                else if (lex.SubType == LexemSubType.preProc)
                {
                    bPreProcReading = true;
                    continue;
                }
                else if (lex.SubType == LexemSubType.programEnd)
                {
                    continue;
                }  

                //если идет чтение цикла со счетчиком
				if (bForLoopReading)
				{
					//если данная лексема равна последней в определении цикла в исходном языке
                    if (lex.SubType == sourceLang.ForLoopLexems[sourceLang.ForLoopLexems.Count - 1].SubType || (lex.Content == "\n" && sourceLang.Name == "Basic"))
					{
						//добавляем все лексемы из буфера в соответствии с необходимой последовательностью
						foreach (Lexem forLex in requiredLang.ForLoopLexems)
						{
							//если нужна переменная, добавляем из буфера
							if (forLex.SubType == LexemSubType.variable)
							{
								for (int i = 0; i < bufForLoopLexems.Count; i++)
								{
									if (bufForLoopLexems[i].SubType == LexemSubType.variable)
									{
                                        if (requiredLang.Name == "Basic")
                                        {
                                            bufBasicForLoopLexems.Push(bufForLoopLexems[i]);
                                        }
										outputFileLexem.Add(bufForLoopLexems[i]);
									}
									break;
								}
							}

							//аналогично для константы
							else if (forLex.SubType == LexemSubType.constant)
							{
								for (int i = 0; i < bufForLoopLexems.Count; i++)
								{
									if (bufForLoopLexems[i].SubType == LexemSubType.constant)
									{
										outputFileLexem.Add(bufForLoopLexems[i]);
										bufForLoopLexems.RemoveAt(i);
										break;
									}
								}
							}

							//иначе просто добавляем необходимую лексему из списка
							else
							{
								foreach (Lexem rlex in requiredLangLex)
								{
									if (forLex.SubType == rlex.SubType)
									{
										outputFileLexem.Add(rlex);
										break;
									}
								}
							}
						}

                        if (requiredLang.Name == "Basic")
                        {
                            basicLoopRotation += 'f';
                        }
                        else
                        {
                            wasReadForLoopCount = -1;
                        }

                        if (sourceLang.Name == "Basic")
                        {
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                        }

                        bufForLoopLexems = new List<Lexem>();						
						bForLoopReading = false;
					}

					continue;
				}
				//если идет чтение условного перехода
				else if (bIfStateReading)
				{
					if (lex.SubType == LexemSubType.blockStart)
					{
						if (requiredLang.BStrictlyStamples)
						{
							outputFileLexem.Insert(outputFileLexem.Count - 2, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
						}

						if (requiredLang.BNeedCloser && !sourceLang.BNeedCloser)
						{
							outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
							outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.algThen, requiredLangLex));
						}

                        if (requiredLang.Name == "Basic")
                        {
                            basicLoopRotation += 'i';
                        }                        
						bIfStateReading = false;
					}                    
                    else if (sourceLang.Name == "Basic")
                    {
                        if (lex.SubType == LexemSubType.algThen)
                        {
                            if (requiredLang.BStrictlyStamples)
                            {
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                            }

                            if (requiredLang.BNeedCloser)
                            {
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.algThen, requiredLangLex));
                            }

                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                            outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                            bIfStateReading = false;
                            continue;
                        }
                        else if (lex.SubType == LexemSubType.logicAssign)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.mathEqual, requiredLangLex));
                            continue;
                        }
                        else if (lex.SubType == LexemSubType.stampleOpenSymbol)
                        {
                            if (bArrayInIORead)
                            {
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                continue;
                            }
                        }
                        else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                        {
                            if (bArrayInIORead)
                            {
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                bArrayInIORead = false;
                                continue;
                            }
                        }
                    }
                    else if (requiredLang.Name == "Basic")
                    {
                        if (lex.SubType == LexemSubType.squareStampleOpen)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                            continue;
                        }
                        else if (lex.SubType == LexemSubType.squareStampleClose)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                            continue;
                        }
                    }
				}
                else if (bWhileLoopReading)
                {
                    if (lex.SubType == LexemSubType.blockStart)
                    {
                        if (requiredLang.BStrictlyStamples)
                        {
                            outputFileLexem.Insert(outputFileLexem.Count - 2, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                        }

                        if (requiredLang.BNeedCloser && !sourceLang.BNeedCloser)
                        {
                            outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.algDo, requiredLangLex));
                        }

                        if (requiredLang.Name == "Basic")
                        {
                            basicLoopRotation += "w";
                        }

                        bWhileLoopReading = false;
                    }
                    else if (lex.SubType == LexemSubType.logicAssign && sourceLang.Name == "Basic")
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.mathEqual, requiredLangLex));
                        continue;
                    }
                    else if (lex.SubType == LexemSubType.algDo && requiredLang.Name == "Basic")
                    {
                        continue;
                    }

                    if (squareStamplesCounter > 0)
                    {
                        switch (lex.SubType)
                        {
                            case LexemSubType.stampleOpenSymbol:
                                if (sourceLang.Name == "Pascal" || sourceLang.Name == "C")
                                {
                                    bNameRead = false;
                                    continue;
                                }
                                break;

                            case LexemSubType.squareStampleOpen:
                                if (squareStamplesCounter <= 1)
                                {
                                    if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                else
                                {
                                    if (requiredLang.Name == "Basic" || requiredLang.Name == "Pascal")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                break;

                            case LexemSubType.squareStampleClose:
                                if (sourceLang.Name != "C")
                                {
                                    bNameRead = false;
                                    squareStamplesCounter = 0;
                                    if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                else continue;
                                break;

                            case LexemSubType.commaSymbol:
                                if (requiredLang.Name == "C")
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                    continue;
                                }
                                break;

                            case LexemSubType.logicAssign:
                                bNameRead = false;
                                if (requiredLang.Name == "Pascal")
                                {
                                    outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                }
                                else
                                {
                                    outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                }
                                break;
                        }
                    }                    
                }
                else if (bRepeatLoopReading)
                {
                    if (bRepeatDefenitonReading && sourceLang.BInversedRepeat != requiredLang.BInversedRepeat)
                    {
                        switch (lex.SubType)
                        {
                            case LexemSubType.mathEqual:                                
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.mathUnequal, requiredLangLex));
                                break;
                            case LexemSubType.mathUnequal:
                                if (requiredLang.Name == "Basic")
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAssign, requiredLangLex));
                                else
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.mathEqual, requiredLangLex));
                                break;
                            case LexemSubType.mathMore:
                                if (requiredLang.Name == "Basic")
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAssign, requiredLangLex));
                                else
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.mathEqual, requiredLangLex));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.mathLess, requiredLangLex));
                                break;
                            case LexemSubType.mathLess:
                                if (requiredLang.Name == "Basic")
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAssign, requiredLangLex));
                                else
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.mathEqual, requiredLangLex));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.mathMore, requiredLangLex));
                                break;
                            case LexemSubType.logicAnd:
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicOr, requiredLangLex));
                                break;
                            case LexemSubType.logicOr:
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAnd, requiredLangLex));
                                break;
                            case LexemSubType.logicNot:
                                continue;
                            default:
                                outputFileLexem.Add(FindLexemWithSubType(lex.SubType, requiredLangLex));
                                break;
                        }

                        continue;
                    }

                    if ((!requiredLang.BStrictlyBlockInRepeat) && (lex.SubType == LexemSubType.blockEnd || lex.SubType == LexemSubType.blockStart))
                    {
                        continue;
                    }                 
                }
                else if (bSubProgHeaderReading)
                {
                    switch (sourceLang.Name)
                    {
                        case "C":
                            if (requiredLang.Name == "Pascal")
                            {
                                if (lex.SubType == LexemSubType.commaSymbol)
                                {
                                    outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    cursorPositionInProc = variableCursorPosition;
                                    bVariableReading = false;
                                    bufVariableLexems.Clear();
                                }
                                else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                                {
                                    if (bVariableReading)
                                    {
                                        outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        bVariableReading = false;
                                        bufVariableLexems.Clear();
                                    }

                                    outputFileLexem.Insert(variableCursorPosition, lex);
                                    bSubProgHeaderReading = false;                                    
                                    cursorPositionInProc = outputFileLexem.Count + 1;
                                }                                
                            }
                            else if (requiredLang.Name == "Basic")
                            {
                                if (lex.SubType == LexemSubType.commaSymbol)
                                {
                                    outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    bVariableReading = false;
                                    cursorPositionInProc = variableCursorPosition;
                                    bufVariableLexems.Clear();
                                }
                                else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                                {
                                    if (bVariableReading)
                                    {
                                        outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        bVariableReading = false;
                                        bufVariableLexems.Clear();
                                    }

                                    outputFileLexem.Insert(variableCursorPosition, lex);
                                    bSubProgHeaderReading = false;
                                    cursorPositionInProc = outputFileLexem.Count + 1;
                                }
                            }
                            break;

                        case "Pascal":
                            if (requiredLang.Name == "C")
                            {
                                if (lex.SubType == LexemSubType.stampleOpenSymbol)
                                {
                                    variableCursorPosition = outputFileLexem.Count;
                                    outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                    currSubProgName = bufVariableLexems[0];
                                    for (int r = 0; r < allVars.Count; r++)
                                    {
                                        if (allVars[r].Content == currSubProgName.Content)
                                        {
                                            allVars.RemoveAt(r);
                                            break;
                                        }
                                    }
                                    bufVariableLexems.Clear();
                                    typeOfCurrentLexem = LexemSubType.any;
                                }
                                else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                                {
                                    if (bufVariableLexems.Count > 0)
                                    {
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);                                                                                
                                        bufVariableLexems.Clear();
                                    }
                                    outputFileLexem.Insert(variableCursorPosition++, lex);
                                    stamplesOpenCounter = 0;
                                    typeOfCurrentLexem = LexemSubType.any;
                                }
                                else if (lex.SubType == LexemSubType.endOfLine)
                                {
                                    if (stamplesOpenCounter == 0)
                                    {
                                        if (typeOfCurrentLexem != LexemSubType.any)
                                        {
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            subProgRead = typeOfCurrentLexem;
                                        }
                                        else
                                        {
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.typeVoid, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            subProgRead = LexemSubType.typeVoid;
                                        }
                                        outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                        bSubProgHeaderReading = false;
                                        cursorPositionInProc = outputFileLexem.Count + 1;
                                    }
                                    else
                                    {
                                        if (bufVariableLexems.Count > 0)
                                        {
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            bufVariableLexems.Clear();
                                        }                                       
                                        
                                        typeOfCurrentLexem = LexemSubType.any;
                                    }
                                }
                            }
                            else if (requiredLang.Name == "Basic")
                            {
                                if (lex.SubType == LexemSubType.stampleOpenSymbol)
                                {
                                    variableCursorPosition = outputFileLexem.Count;
                                    outputFileLexem.Add(bufVariableLexems[0]);
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                    currSubProgName = bufVariableLexems[0];
                                    for (int r = 0; r < allVars.Count; r++)
                                    {
                                        if (allVars[r].Content == currSubProgName.Content)
                                        {
                                            allVars.RemoveAt(r);
                                            break;
                                        }
                                    }
                                    bufVariableLexems.Clear();
                                    typeOfCurrentLexem = LexemSubType.any;
                                }
                                else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                                {
                                    if (bufVariableLexems.Count > 0)
                                    {
                                        outputFileLexem.Add(bufVariableLexems[0]);
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));                                        
                                        bufVariableLexems.Clear();
                                    }
                                    outputFileLexem.Add(lex);
                                    stamplesOpenCounter = 0;
                                    typeOfCurrentLexem = LexemSubType.any;
                                }
                                else if (lex.SubType == LexemSubType.endOfLine)
                                {
                                    if (stamplesOpenCounter == 0)
                                    {
                                        if (typeOfCurrentLexem != LexemSubType.any)
                                        {
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.function, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            subProgRead = typeOfCurrentLexem;
                                        }
                                        else
                                        {
                                            outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.procedure, requiredLangLex));
                                            outputFileLexem.Insert(variableCursorPosition, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            subProgRead = LexemSubType.typeVoid;
                                        }
                                        
                                        bSubProgHeaderReading = false;
                                        cursorPositionInProc = outputFileLexem.Count + 1;
                                    }
                                    else
                                    {
                                        if (bufVariableLexems.Count > 0)
                                        {
                                            outputFileLexem.Add(bufVariableLexems[0]);
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                            bufVariableLexems.Clear();
                                        }
                                        typeOfCurrentLexem = LexemSubType.any;
                                    }
                                }
                            }
                            break;

                        case "Basic":
                            if (requiredLang.Name == "Pascal")
                            {
                                if (lex.SubType == LexemSubType.commaSymbol)
                                {
                                    outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    cursorPositionInProc = variableCursorPosition;
                                    bufVariableLexems.Clear();
                                }
                                else if (lex.SubType == LexemSubType.stampleOpenSymbol)
                                {
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                    outputFileLexem.Insert(variableCursorPosition++, lex);
                                    bVariableReading = false;
                                    bufVariableLexems.Clear();
                                }
                                else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                                {
                                    if (bufVariableLexems.Count > 0)
                                    {
                                        outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                        outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        bVariableReading = false;
                                        bufVariableLexems.Clear();
                                    }

                                    outputFileLexem.Insert(variableCursorPosition++, lex);                                                        
                                }
                                else if (lex.SubType == LexemSubType.carriageReturn)
                                {
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, lex);
                                    outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                                    outputFileLexem.Insert(variableCursorPosition++, lex);
                                    cursorPositionInProc = outputFileLexem.Count + 1;
                                    bSubProgHeaderReading = false;
                                    subProgRead = typeOfCurrentLexem;
                                }                                
                            }
                            else if (requiredLang.Name == "C")
                            {
                                if (lex.SubType == LexemSubType.commaSymbol)
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Add(bufVariableLexems[0]);
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    bufVariableLexems.Clear();

                                    typeOfCurrentLexem = LexemSubType.any;
                                    bufVariableLexems.Clear();
                                }
                                else if (lex.SubType == LexemSubType.stampleOpenSymbol)
                                {
                                    cursorPositionInProc = outputFileLexem.Count;
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    outputFileLexem.Add(bufVariableLexems[0]);
                                    outputFileLexem.Add(lex);
                                    bVariableReading = false;
                                    bufVariableLexems.Clear();
                                    
                                    typeOfCurrentLexem = LexemSubType.any;                                    
                                }
                                else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                                {
                                    if (bufVariableLexems.Count > 0)
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Add(bufVariableLexems[0]);
                                        typeOfCurrentLexem = LexemSubType.any;
                                        bufVariableLexems.Clear();
                                    }
                                    outputFileLexem.Add(lex);                                    
                                }
                                else if (lex.SubType == LexemSubType.carriageReturn)
                                {
                                    if (typeOfCurrentLexem != LexemSubType.any)
                                    {
                                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                        outputFileLexem.Insert(cursorPositionInProc, FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        typeOfCurrentLexem = LexemSubType.any;
                                    }
                                    else
                                    {
                                        outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                        outputFileLexem.Insert(cursorPositionInProc, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        typeOfCurrentLexem = LexemSubType.any;
                                    }
                                    outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                                    outputFileLexem.Add(new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                    bSubProgHeaderReading = false;
                                    if (typeOfCurrentLexem == LexemSubType.any)
                                        subProgRead = LexemSubType.typeVoid;
                                    else
                                        subProgRead = typeOfCurrentLexem;
                                }
                            }
                            break;
                    }

                    continue;
                }
                else if (bNameRead)
                {
                    if (squareStamplesCounter > 0)
                    {
                        switch (lex.SubType)
                        {
                            case LexemSubType.stampleOpenSymbol:
                                if (sourceLang.Name == "Pascal" || sourceLang.Name == "C")
                                {
                                    bNameRead = false;
                                    continue;
                                }                                
                                break;

                            case LexemSubType.squareStampleOpen:
                                if (squareStamplesCounter <= 1)
                                {
                                    if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                else
                                {
                                    if (requiredLang.Name == "Basic" || requiredLang.Name == "Pascal")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                break;                            

                            case LexemSubType.squareStampleClose:
                                if (sourceLang.Name != "C")
                                {
                                    bNameRead = false;
                                    squareStamplesCounter = 0;
                                    if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                else continue;
                                break;

                            case LexemSubType.commaSymbol:
                                if (requiredLang.Name == "C")
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                    continue;
                                }
                                break;

                            case LexemSubType.logicAssign:
                                bNameRead = false;
                                if (requiredLang.Name == "Pascal")
                                {
                                    outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                }
                                else
                                {
                                    outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                }
                                break;
                        }                        
                    }
                    else
                    {
                        switch (lex.SubType)
                        {
                            case LexemSubType.stampleOpenSymbol:
                                if (sourceLang.Name == "Basic" && bArrayInIORead)
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                    continue;
                                }
                                break;

                            case LexemSubType.stampleCloseSymbol:
                                bNameRead = false;
                                squareStamplesCounter = 0;
                                if (sourceLang.Name == "Basic" && bArrayInIORead)
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                    bArrayInIORead = false;
                                    continue;
                                }
                                else
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                }
                                continue;
                        }
                    }
                }
				else if (bVariableReading)
				{
                    if (bConstValueReading)
                    {
                        if (lex.SubType == LexemSubType.blockStart)
                        {
                            bufConstValueLexems.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                        }
                        else if (lex.SubType == LexemSubType.blockEnd)
                        {
                            bufConstValueLexems.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                        }
                        else if (lex.SubType == LexemSubType.stampleOpenSymbol)
                        {
                            bufConstValueLexems.Add(FindLexemWithSubType(LexemSubType.blockStart, requiredLangLex));
                        }
                        else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                        {
                            bufConstValueLexems.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                        }
                        else
                        {
                            bufConstValueLexems.Add(FindLexemWithSubType(lex.SubType, requiredLangLex));
                        }

                        continue;
                    }

				    if (sourceLang.Name == "C")
				    {
				        if (lex.SubType == LexemSubType.logicAssign)
				        {
				            bConstValueReading = true;
				        }
                        else if (lex.SubType == LexemSubType.squareStampleOpen)
                        {
                            bArrayReading = true;
                            arrayCursorPosition = cursorPositionInProc;
                            bVariableReading = false;
                        }
                        else if (lex.SubType == LexemSubType.stampleOpenSymbol)
                        {
                            if (!bMainConstructionReading)
                            {                               
                                bVariableReading = false;
                                bSubProgHeaderReading = true;
                                stamplesOpenCounter = 1;

                                subProgRead = typeOfCurrentLexem;

                                if (typeOfCurrentLexem == LexemSubType.typeVoid)
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.procedure, requiredLangLex));
                                    variableCursorPosition = outputFileLexem.Count;
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                }
                                else
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.function, requiredLangLex));
                                    variableCursorPosition = outputFileLexem.Count;
                                    if (requiredLang.Name == "Pascal")
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.colonSymbol, requiredLangLex));
                                    else if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.logicAs, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                    }
                                    
                                    outputFileLexem.Add(FindLexemWithSubType(typeOfCurrentLexem, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                }

                                outputFileLexem.Insert(variableCursorPosition++, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                outputFileLexem.Insert(variableCursorPosition++, bufVariableLexems[0]);
                                outputFileLexem.Insert(variableCursorPosition++, lex);
                                cursorPositionInProc = variableCursorPosition;
                                currSubProgName = bufVariableLexems[0];
                                for (int r = 0; r < allVars.Count; r++)
                                {
                                    if (allVars[r].Content == currSubProgName.Content)
                                    {
                                        allVars.RemoveAt(r);
                                        break;
                                    }
                                }
                                bufVariableLexems.Clear();
                            }                            
                        }                        
				    }
                    else if (sourceLang.Name == "Pascal")
                    {
                        if (lex.SubType == LexemSubType.blockStart)
                        {
                            bVariableReading = false;
                            bConstReading = false;
                            outputFileLexem.Add(FindLexemWithSubType(lex.SubType, requiredLangLex));
                        }
                        else if (lex.SubType == LexemSubType.mathEqual)
                        {
                            bConstValueReading = true;
                        }
                        else if (lex.SubType == LexemSubType.squareStampleOpen)
                        {
                            bArrayReading = true;
                            arrayCursorPosition = cursorPositionInProc;
                            bVariableReading = false;
                        }
                    }
                    else if (sourceLang.Name == "Basic")
                    {
                        if (lex.SubType == LexemSubType.stampleOpenSymbol)
                        {
                            bArrayReading = true;
                            arrayCursorPosition = cursorPositionInProc;
                            bVariableReading = false;
                        }
                    }

                    continue;
				}
                else if (bArrayReading)
                {
                    if (lex.SubType == LexemSubType.commaSymbol || (lex.SubType == LexemSubType.squareStampleOpen && squareStamplesCounter > 1))
                    {
                        if (!bCommaInArrayRead)
                        {
                            switch (requiredLang.Name)
                            {
                                case "C":
                                    {
                                        for (int i = 0; i < bufVariableLexems.Count; i++)
                                        {
                                            outputFileLexem.Insert(arrayCursorPosition++, bufVariableLexems[i]);
                                            if (i != bufVariableLexems.Count - 1)
                                            {
                                                outputFileLexem.Insert(arrayCursorPosition++,
                                                                       FindLexemWithSubType(LexemSubType.commaSymbol,
                                                                                            requiredLangLex));
                                            }
                                        }
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.squareStampleOpen,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[1]);
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.squareStampleClose,
                                                                                    requiredLangLex));
                                    }
                                    break;

                                case "Pascal":
                                    {
                                        for (int i = 0; i < bufVariableLexems.Count; i++)
                                        {
                                            outputFileLexem.Insert(arrayCursorPosition++, bufVariableLexems[i]);
                                            if (i != bufVariableLexems.Count - 1)
                                            {
                                                outputFileLexem.Insert(arrayCursorPosition++,
                                                                       FindLexemWithSubType(LexemSubType.commaSymbol,
                                                                                            requiredLangLex));
                                            }
                                        }
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.colonSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.arrayType,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.squareStampleOpen,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.arrayRange,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[bufArrayConstLexems.Count-1]);
                                    }
                                    break;

                                case "Basic":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                                  FindLexemWithSubType(LexemSubType.varBlock,
                                                                                       requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.logicAs,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        for (int i = 0; i < bufVariableLexems.Count; i++)
                                        {
                                            outputFileLexem.Insert(arrayCursorPosition++, bufVariableLexems[i]);
                                            if (i != bufVariableLexems.Count - 1)
                                            {
                                                outputFileLexem.Insert(arrayCursorPosition++,
                                                                       FindLexemWithSubType(LexemSubType.commaSymbol,
                                                                                            requiredLangLex));
                                            }
                                        }
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));                                        
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.stampleOpenSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.algTo,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[0]);
                                    }
                                    break;
                            }

                            bCommaInArrayRead = true;
                        }
                        else
                        {
                            switch (requiredLang.Name)
                            {
                                case "C":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.squareStampleOpen,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[1]);
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.squareStampleClose,
                                                                                    requiredLangLex));
                                    }
                                    break;

                                case "Pascal":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.commaSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.arrayRange,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[bufArrayConstLexems.Count - 1]);
                                    }
                                    break;

                                case "Basic":
                                    {
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.commaSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               new Lexem("1", LexemType.Variable, LexemSubType.constant));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.algTo,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++,
                                                               FindLexemWithSubType(LexemSubType.spaceSymbol,
                                                                                    requiredLangLex));
                                        outputFileLexem.Insert(arrayCursorPosition++, bufArrayConstLexems[0]);
                                    }
                                    break;
                            }
                        }

                        bufArrayConstLexems.Clear();
                    }

                    continue;
                }
                else if (bInputReading)
                {               
                    if (lex.SubType == LexemSubType.stringOpen)
                    {
                        if (bQuotesRead)
                        {
                            if (requiredLang.Name == "C")
                                quoteInIOCursor = outputFileLexem.Count;
                            bQuotesRead = false;
                        }
                        else
                            bQuotesRead = true;
                    }
                    else if (lex.SubType == LexemSubType.stampleOpenSymbol)
                    {
                        if (bArrayInIORead && sourceLang.Name == "Basic")
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                            continue;
                        }
                    }
                    else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                    {
                        if (bArrayInIORead && sourceLang.Name == "Basic")
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                            bArrayInIORead = false;
                            continue;
                        }
                    }

                    if (requiredLang.Name == "Basic")
                    {
                        if (lex.SubType == LexemSubType.stampleOpenSymbol && stamplesOpenCounter < 2)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            continue;
                        }
                        else if (lex.SubType == LexemSubType.stampleCloseSymbol && stamplesCloseCounter < 2)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            continue;
                        }
                    }

                    if (sourceLang.Name == "C")
                    {
                        if (bQuotesRead)
                            continue;
                    }

                    if (squareStamplesCounter > 0)
                    {
                        switch (lex.SubType)
                        {
                            case LexemSubType.stampleOpenSymbol:
                                if (sourceLang.Name == "Pascal" || sourceLang.Name == "C")
                                {
                                    bNameRead = false;
                                    continue;
                                }
                                break;

                            case LexemSubType.squareStampleOpen:
                                if (squareStamplesCounter <= 1)
                                {
                                    if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                else
                                {
                                    if (requiredLang.Name == "Basic" || requiredLang.Name == "Pascal")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                break;

                            case LexemSubType.squareStampleClose:
                                if (sourceLang.Name != "C")
                                {
                                    bNameRead = false;
                                    squareStamplesCounter = 0;
                                    if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                else continue;
                                break;

                            case LexemSubType.commaSymbol:
                                if (requiredLang.Name == "C")
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                    continue;
                                }
                                break;

                            case LexemSubType.logicAssign:
                                bNameRead = false;
                                if (requiredLang.Name == "Pascal")
                                {
                                    outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                }
                                else
                                {
                                    outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                }
                                break;
                        }
                    }                    
                }
                else if (bOutputReading)
                {
                    if (lex.SubType == LexemSubType.stringOpen)
                    {
                        if (bQuotesRead)
                        {
                            if (requiredLang.Name == "C")
                                quoteInIOCursor = outputFileLexem.Count;
                            bQuotesRead = false;
                        }
                        else
                            bQuotesRead = true;
                    }
                    else if (lex.SubType == LexemSubType.stampleOpenSymbol)
                    {
                        if (bArrayInIORead && sourceLang.Name == "Basic")
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                            continue;
                        }
                    }
                    else if (lex.SubType == LexemSubType.stampleCloseSymbol)
                    {
                        if (bArrayInIORead && sourceLang.Name == "Basic")
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                            bArrayInIORead = false;
                            continue;
                        }
                    }

                    if (requiredLang.Name == "Basic")
                    {
                        if (lex.SubType == LexemSubType.stampleOpenSymbol && stamplesOpenCounter < 2)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            continue;
                        }
                        else if (lex.SubType == LexemSubType.stampleCloseSymbol && stamplesCloseCounter < 2)
                        {
                            outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            continue;
                        } 
                        else if (lex.SubType == LexemSubType.commaSymbol)
                        {
                            if (!bQuotesRead)
                            {
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                outputFileLexem.Add(new Lexem(";", LexemType.Divider, LexemSubType.any));
                                outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                            }
                            continue;
                        }
                    }

                    if (squareStamplesCounter > 0)
                    {
                        switch (lex.SubType)
                        {
                            case LexemSubType.stampleOpenSymbol:
                                if (sourceLang.Name == "Pascal" || sourceLang.Name == "C")
                                {
                                    bNameRead = false;
                                    continue;
                                }
                                break;

                            case LexemSubType.squareStampleOpen:
                                if (squareStamplesCounter <= 1)
                                {
                                    if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleOpenSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                else
                                {
                                    if (requiredLang.Name == "Basic" || requiredLang.Name == "Pascal")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.commaSymbol, requiredLangLex));
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                break;

                            case LexemSubType.squareStampleClose:
                                if (sourceLang.Name != "C")
                                {
                                    bNameRead = false;
                                    squareStamplesCounter = 0;
                                    if (requiredLang.Name == "Basic")
                                    {
                                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                        continue;
                                    }
                                }
                                else continue;
                                break;

                            case LexemSubType.commaSymbol:
                                if (requiredLang.Name == "C")
                                {
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.squareStampleOpen, requiredLangLex));
                                    continue;
                                }
                                break;

                            case LexemSubType.logicAssign:
                                bNameRead = false;
                                if (requiredLang.Name == "Pascal")
                                {
                                    outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.squareStampleClose, requiredLangLex));
                                }
                                else
                                {
                                    outputFileLexem.Insert(outputFileLexem.Count - 1, FindLexemWithSubType(LexemSubType.stampleCloseSymbol, requiredLangLex));
                                }
                                break;
                        }
                    }                    
                }

			    outputFileLexem.Add(FindLexemWithSubType(lex.SubType, requiredLangLex));
			}

            int op = 1;
            for (int i = 1; i < outputFileLexem.Count; i++)
            {           
                if (outputFileLexem[op].Content == "" || outputFileLexem[op].Content == null)
                {
                    outputFileLexem.RemoveAt(op);
                    continue;
                }

                if ((outputFileLexem[op].Content == "\n" || outputFileLexem[op].Content == " ") && outputFileLexem[op - 1].Content == "\n")
                {
                    outputFileLexem.RemoveAt(op);
                    continue;
                }                

                op++;
            }

            switch (requiredLang.Name)
            {
                case "Pascal":     
                    outputFileLexem.Insert(0, FindLexemWithSubType(LexemSubType.programStart, requiredLangLex));
                    outputFileLexem.Insert(1, FindLexemWithSubType(LexemSubType.spaceSymbol, requiredLangLex));
                    outputFileLexem.Insert(2, new Lexem("program_1", LexemType.Variable, LexemSubType.any));
                    outputFileLexem.Insert(3, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));              
                    outputFileLexem.Insert(6, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                    if (sourceLang.Name == "Basic")
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                    }
                    outputFileLexem.Add(FindLexemWithSubType(LexemSubType.programEnd, requiredLangLex));

                    int bufEndIndexer = 0;
                    bool bNeedInsertEndOfLine = false;
                    bool bVarInserted = false;
                    bool bFunctionRead = false;
                    int cursorToVar = 0;
                    for (int i = 0; i < outputFileLexem.Count; i++)
                    {
                        if (outputFileLexem[i].SubType == LexemSubType.blockEnd)
                        {
                            if (bNeedInsertEndOfLine)
                            {
                                outputFileLexem.Insert(bufEndIndexer, FindLexemWithSubType(LexemSubType.endOfLine, requiredLangLex));
                                i++;
                            }
                            else
                                bNeedInsertEndOfLine = true;

                            bufEndIndexer = i + 1;
                        }
                        else if (outputFileLexem[i].SubType == LexemSubType.algElse)
                        {
                            bNeedInsertEndOfLine = false;
                        }
                        else if (outputFileLexem[i].SubType == LexemSubType.procedure || outputFileLexem[i].SubType == LexemSubType.function)
                        {
                            bFunctionRead = true;
                        }
                        else if (outputFileLexem[i].SubType == LexemSubType.endOfLine)
                        {
                            if (bFunctionRead)
                            {
                                bFunctionRead = false;
                                bVarInserted = false;   
                            }                            
                        }

                        if (!bVarInserted)
                        {
                            if (outputFileLexem[i].SubType == LexemSubType.variable)
                            {
                                cursorToVar = i;
                                if (outputFileLexem[i + 1].SubType == LexemSubType.colonSymbol)
                                {
                                    outputFileLexem.Insert(cursorToVar++, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                    outputFileLexem.Insert(cursorToVar++, FindLexemWithSubType(LexemSubType.varBlock, requiredLangLex));
                                    outputFileLexem.Insert(cursorToVar++, new Lexem("\n", LexemType.Divider, LexemSubType.carriageReturn));
                                    bVarInserted = true;
                                }
                            }
                        }
                    }                    
                    break;
                case "C":
                    for (int i = 0; i < outputFileLexem.Count - 1; i++)
                    {
                        if ((outputFileLexem[i].SubType == LexemSubType.carriageReturn ||
                        outputFileLexem[i].Content == "\n" ||
                        outputFileLexem[i].SubType == LexemSubType.any ||
                        outputFileLexem[i].SubType == LexemSubType.blockEnd) && outputFileLexem[i + 1].SubType == LexemSubType.endOfLine)
                        {
                            outputFileLexem.RemoveAt(i + 1);
                        }
                    }

                    if (sourceLang.Name == "Basic")
                    {
                        outputFileLexem.Add(FindLexemWithSubType(LexemSubType.blockEnd, requiredLangLex));
                    }
                    break;
            }            
            
			return outputFileLexem;
		}

		//функция поиска лексемы с именем name среди коллекции лексем sourceLangLex
		static Lexem FindLexemByName(string name, List<Lexem> sourceLangLex)
		{
			foreach (Lexem lex in sourceLangLex)
			{
				if (name.ToLower() == lex.Content.ToLower())
				{
					return lex;
				}
			}

			//если переданная строка является числом, возвращаем константу
			if (char.IsDigit(name[0]))
			{
				return new Lexem(name, LexemType.Variable, LexemSubType.constant);
			}

			return new Lexem(name, LexemType.Variable, LexemSubType.variable);
		}

		//вторая версия той же функции
		static Lexem FindLexemByName(char name, List<Lexem> sourceLangLex)
		{
			string strName = null;
			strName += name;

			return FindLexemByName(strName, sourceLangLex);
		}

		//функция проверки наличия лексемы с именем name среди коллекции sourceLangLex
		static bool CheckLexemContaining(char name, List<Lexem> sourceLangLex)
		{
			string str = null;
			str += name;

			foreach (Lexem lex in sourceLangLex)
			{
				if (str == lex.Content)
				{
					return true;
				}
			}
			return false;
		}

		//вторая версия той же функции
		static bool CheckLexemContaining(string name, List<Lexem> sourceLangLex)
		{
			foreach (Lexem lex in sourceLangLex)
			{
				if (name == lex.Content)
				{
					return true;
				}
			}
			return false;
		}

		static bool CheckLexemContaining(LexemSubType lst, List<Lexem> sourceLangLex)
		{
			foreach (Lexem lex in sourceLangLex)
			{
				if (lst == lex.SubType)
				{
					return true;
				}
			}
			return false;
		}

		//функция поиска лексемы в коллекции по подтипу
        static Lexem FindLexemWithSubType(LexemSubType lst, List<Lexem> inputLexem)
        {
            foreach (Lexem lex in inputLexem)
            {
                if (lst == lex.SubType)
                    return lex;
            }
            return new Lexem("", LexemType.Variable, LexemSubType.any);
        }
    }
}
